/*
 *  Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 *  Use of this source code is governed by a MIT license that can be found in the LICENSE file
 */

package com.netease.nos;

import android.content.Context;

import com.netease.nos.model.NosUploadState;
import com.netease.nos.model.SafeResult;
import com.netease.nos.thread.HandlerUtils;
import com.netease.yunxin.nos.sdk.NosComponent;
import com.netease.yunxin.nos.sdk.NosFacade;
import com.netease.yunxin.nos.sdk.NosToken;
import com.netease.yunxin.nos.sdk.UploadCallback;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * NosPlugin 提供文件上传到云信云盘功能
 */
public class NosPlugin implements FlutterPlugin, MethodCallHandler {

    private MethodChannel channel;

    private Context context;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "nos_sdk");
        channel.setMethodCallHandler(this);
        context = flutterPluginBinding.getApplicationContext();
    }

    private static MapBuilder map() {
        return new MapBuilder();
    }

    public static class MapBuilder {

        private Map<String, Object> map;

        public MapBuilder(int size) {
            map = new HashMap<>(size);
        }

        private MapBuilder() {
            map = new HashMap<>();
        }

        public MapBuilder put(String key, Object value) {
            map.put(key, value);
            return this;
        }

        private Map<String, Object> build() {
            return map;
        }
    }

    @Override
    public void onMethodCall(@NonNull final MethodCall call, @NonNull final Result unSafeResult) {
        SafeResult result = new SafeResult(unSafeResult);
        if (call.method.equals("uploadSync")) {
            final String sdkType = call.argument("sdkType");
            final String sdkVersionName = call.argument("sdkVersionName");
            String path = call.argument("path");
            final List<String> filePath = Collections.singletonList(path);
            final String appKey = call.argument("appKey");
            NosComponent data = new NosComponent.Builder(context, appKey).build();
            NosFacade.setupOnce(data, true);
            HandlerUtils.runOnOtherThread(() -> {
                NosFacade.uploadSync(sdkType, sdkVersionName, filePath, path, null, new UploadCallback() {

                    @Override
                    public void onProgress(Object key1, long current, long total) {
                        MapBuilder builder = map().put("receiptId", call.argument("receiptId"));
                        NosUploadState nosUploadState = NosUploadState.progressState((String) key1, current, total);
                        runOnUiThreadUploadState(builder, nosUploadState);
                    }
                    @Override
                    public void onSuccess(Object key1, String url) {
                        MapBuilder builder = map().put("receiptId", call.argument("receiptId"));
                        NosUploadState nosUploadState = NosUploadState.successState((String) key1, url);
                        runOnUiThreadUploadState(builder, nosUploadState);

                    }
                    @Override
                    public void onFailure(Object key1, int code, String response) {
                        MapBuilder builder = map().put("receiptId", call.argument("receiptId"));
                        NosUploadState nosUploadState = NosUploadState.failState((String) key1, code, response);
                        runOnUiThreadUploadState(builder, nosUploadState);
                    }
                    @Override
                    public void onCanceled(Object key1) {
                        MapBuilder builder = map().put("receiptId", call.argument("receiptId"));
                        NosUploadState nosUploadState = NosUploadState.cancelState((String) key1);
                        runOnUiThreadUploadState(builder, nosUploadState);
                    }
                });
                result.success(0);
            });
        } else {
            result.notImplemented();
        }
    }
    private void runOnUiThreadUploadState(final MapBuilder builder, final NosUploadState nosUploadState) {
        HandlerUtils.runOnUiThread(() -> channel.invokeMethod("onUploadSync", nosUploadState.toJson(builder).build()));
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
