/*
 *  Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 *  Use of this source code is governed by a MIT license that can be found in the LICENSE file
 */

package com.netease.nos.thread;

import android.os.Process;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by hzsunyj on 2020/6/11.
 */
public class ExecutorUtils {

    private static final int CORE_POOL_SIZE = Runtime.getRuntime().availableProcessors();

    private static final int MAXIMUM_POOL_SIZE = 16;

    private static final int KEEP_ALIVE_SECONDS = 3;


    private static final ThreadFactory threadFactory = new ThreadFactory() {

        private final AtomicInteger mCount = new AtomicInteger(1);

        public Thread newThread(Runnable r) {
            Thread thread = new Thread(r, "Nos #" + mCount.getAndIncrement());
            Process.setThreadPriority(Process.THREAD_PRIORITY_BACKGROUND);
            return thread;
        }
    };

    private static ExecutorService executor = new ThreadPoolExecutor(CORE_POOL_SIZE, MAXIMUM_POOL_SIZE,
                                                                     KEEP_ALIVE_SECONDS, TimeUnit.SECONDS,
                                                                     new SynchronousQueue<Runnable>(), threadFactory);

    /// can shut down
    public static ExecutorService getExecutor() {
        return executor;
    }
}
