//
//  NIMNOSUtil.m
//  YixinCall
//
//  Created by amao on 12/26/14.
//  Copyright (c) 2014 amao. All rights reserved.
//

#import "NIMFNOSUtil.h"
#import <CommonCrypto/CommonDigest.h>

#ifndef NIM_MAC
@import MobileCoreServices;
#else
@import CoreServices;
#endif

@implementation NIMFNOSUtil
+ (NSString *)mimeType:(NSString *)filepath
{
    NSString *mimeType = nil;
    NSString *extension = [filepath pathExtension];
    if ([extension length])
    {
        CFStringRef UTI = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (__bridge CFStringRef)extension, NULL);
        if (UTI)
        {
            CFStringRef mimeTypeRef = UTTypeCopyPreferredTagWithClass (UTI, kUTTagClassMIMEType);
            if (!mimeTypeRef)
            {
                //某些机器无法解析某些后缀,加特别判断
                if ([extension isEqualToString:@"aac"])
                {
                    mimeType = @"audio/aac";
                }
            }
            else
            {
                mimeType = CFBridgingRelease(mimeTypeRef);
            }
            CFRelease(UTI);
        }
    }
    if (!mimeType)
    {
        mimeType = @"application/octet-stream";
    }
    return mimeType;
}

+ (NSString *)fileMD5:(NSString *)filepath
{
    NSString *md5 = nil;
    
    if ([[NSFileManager defaultManager] fileExistsAtPath:filepath])
    {
        unsigned long long fileSize = [self fileSize:filepath];
        if(fileSize > 200 * 1024 * 1024) { 
            md5 = [self bigFileMD5:filepath];
        }else {
            NSData *data = [NSData dataWithContentsOfFile:filepath];
            md5 =  [self nim_md5:data];
        }
    }
    return md5;
}

+ (NSString *)nim_md5:(NSData *)data
{
    unsigned char r[CC_MD5_DIGEST_LENGTH];
    CC_MD5([data bytes], (CC_LONG)[data length], r);
    return  [NSString stringWithFormat:@"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
            r[0], r[1], r[2], r[3], r[4], r[5], r[6], r[7], r[8], r[9], r[10], r[11], r[12], r[13], r[14], r[15]];
}

+ (void)fileMD5:(NSString *)filepath completion:(void(^)(NSString *MD5))completion;
{
    dispatch_block_t block = ^(){
        NSString *md5 = [self fileMD5:filepath];
        if(completion) {
            completion(md5);
        }
    };
    
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0);
    dispatch_async(queue,block);
}

+ (NSString *)bigFileMD5:(NSString *)filePath
{
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForReadingAtPath:filePath];
    if(!fileHandle) {
        return nil;
    }
    
    CC_MD5_CTX ctx;
    CC_MD5_Init(&ctx);
    
    BOOL done = NO;
    NSData *data = nil;
    while (!done) {
        @autoreleasepool {
            data = [fileHandle readDataOfLength:256];
            if([data length]) {
                CC_MD5_Update(&ctx, [data bytes], (CC_LONG)[data length]);
            }else {
                done = YES;
            }
        }
    };
    unsigned char digest[CC_MD5_DIGEST_LENGTH];
    CC_MD5_Final(digest, &ctx);
    NSString *result = [NSString stringWithFormat:@"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
                        digest[0],digest[1],
                        digest[2],digest[3],
                        digest[4],digest[5],
                        digest[6],digest[7],
                        digest[8],digest[9],
                        digest[10],digest[11],
                        digest[12],digest[13],
                        digest[14],digest[15]
                        ];
    return result;
}


+ (unsigned long long)fileSize:(NSString *)filepath
{
    unsigned long long fileSize = 0;
    if ([[NSFileManager defaultManager] fileExistsAtPath:filepath])
    {
        NSDictionary *attributes = [[NSFileManager defaultManager] attributesOfItemAtPath:filepath
                                                                                    error:nil];
        id item = [attributes objectForKey:NSFileSize];
        fileSize = [item isKindOfClass:[NSNumber class]] ? [item unsignedLongLongValue] : 0;
    }
    return fileSize;

}

@end


