//
//  NIMFNosHandler.m
//  nos
//
//  Created by 张根宁 on 2020/6/15.
//

#import "NIMFNosHandler.h"
#import "NIMFNosUploadRequest.h"
#import "NIMFNOSUploader.h"
#import "NIMFNosTokenManager.h"

@interface NIMFNosHandler ()

@property (nonatomic, weak) FlutterMethodChannel *channel;

@property (nonatomic, strong) NIMFNOSUploader *uploader;

@end

@implementation NIMFNosHandler

- (instancetype)initWithChannel:(FlutterMethodChannel *)channel {
    if (self = [super init]) {
        _channel = channel;
    }
    return self;
}

- (BOOL)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    BOOL handled = NO;
    
    if ([call.method isEqualToString:@"uploadSync"]) {
        [self didUploadSync:call.arguments result:result];
        handled = YES;
    }
    
    return handled;
}

#pragma mark - Private
- (void)didRequestUploadToken:(NIMFNosUploadRequest *)uploadRequest
                   completion:(NIMFGetTokenCompletion)block {
    NIMFGetNosTokenTask *task = [[NIMFGetNosTokenTask alloc] init];
    task.appkey = uploadRequest.appKey;
    task.sdktype = uploadRequest.sdkType;
    task.sdkver = uploadRequest.sdkVersionName;
    task.completion = block;
    [task resume];
}

#pragma mark - Receive
- (void)didUploadSync:(NSDictionary *)reqeustDictionary result:(FlutterResult)result {
    NIMFNosUploadRequest *request = [[NIMFNosUploadRequest alloc] initWithDictionary:reqeustDictionary];
    NIMFNosUploadResponse *response = [[NIMFNosUploadResponse alloc] initWithRequest:request];
    
    __weak typeof(self) weakSelf = self;
    
    //请求Token
    [self didRequestUploadToken:request
                     completion:^(NSError * _Nullable error, NIMFNosToken * _Nullable token) {
        
        if (error) {
            [response setError:error];
            [weakSelf sendOnUploadSyncResponse:response];
        } else {//上传
            [weakSelf.uploader upload:request.path
                                token:token
                             progress:^(float percent) {
                response.state = NIMFNosUploadStateLoading;
                response.percent = percent;
                [weakSelf sendOnUploadSyncResponse:response];
            } completion:^(NSError *error, NSString *url) {
                NIMFNosUploadResponse *res = [[NIMFNosUploadResponse alloc] initWithRequest:request];
                if (error) {
                    if (error.code == 0) { //cancel
                        res.state = NIMFNosUploadStateCancel;
                    } else {
                        [response setError:error];
                    }
                } else {
                    res.state = NIMFNosUploadStateOK;
                    res.url = url;
                }
                [weakSelf sendOnUploadSyncResponse:res];
            }];
        }
    }];
    result(nil);
}

#pragma mark - Getter
- (NIMFNOSUploader *)uploader {
    if (!_uploader) {
        _uploader = [[NIMFNOSUploader alloc] init];
    }
    return _uploader;
}

#pragma mark - Send
- (void)sendOnUploadSyncResponse:(NIMFNosUploadResponse *)response {
    NSDictionary *arguments = [response toDictionary];
    [_channel invokeMethod:@"onUploadSync" arguments:arguments];
}

@end
