//
//  NIMFNosTokenManager.m
//  nos
//
//  Created by 张根宁 on 2020/6/16.
//

#import "NIMFNosTokenManager.h"
#import "NSString+NIMF.h"
#import "NIMFAFNetworking.h"
#import "NIMFNosToken.h"
#import <CommonCrypto/CommonDigest.h>

@implementation NIMFNosTokenManager

+ (void)request:(id<NIMFNetworkProtocol>)task {
    NSURLRequest *req = [task request];
    NSURLSession *session = [NSURLSession sharedSession];
    NSURLSessionTask *sessionTask = [session dataTaskWithRequest:req
                                               completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable connectionError) {
        id jsonObject = nil;
        NSError *error = connectionError;
        if (connectionError == nil &&
            [response isKindOfClass:[NSHTTPURLResponse class]] &&
            [(NSHTTPURLResponse *)response statusCode] == 200)
        {
            if (data)
            {
                jsonObject = [NSJSONSerialization JSONObjectWithData:data
                                                             options:0
                                                               error:&error];
            }
            else
            {
                error = [NSError errorWithDomain:@"ntes domain"
                                            code:-1
                                        userInfo:@{@"description" : @"invalid data"}];
                
            }
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [task response:jsonObject error:error];
        });
    }];
    [sessionTask resume];
}

@end


@interface NIMFGetNosTokenTask ()

@property (nonatomic, copy) NSString *platform;
@property (nonatomic, copy) NSString *deviceInfo;
@property (nonatomic, copy) NSString *ext;

@end

@implementation NIMFGetNosTokenTask

- (instancetype)init {
    if (self = [super init]) {
        _sdktype = @"meeting";
        _sdkver = @"0.5.0";
        _platform = @"iOS";
        _deviceInfo = [NSString nimf_deveicInfo];
        _ext = @"zip";
    }
    return self;
}

- (NSString *)path {
    return @"https://statistic.live.126.net/sdklog/getToken";
}

- (NSMutableDictionary *)dictionaryValue {
    NSMutableDictionary *dic = [NSMutableDictionary dictionary];
    dic[@"appKey"] = _appkey ?: @"";
    dic[@"sdktype"] = _sdktype ?: @"";
    dic[@"sdkver"] = _sdkver ?: @"";
    dic[@"platform"] = _platform ?: @"";
    dic[@"deviceInfo"] = _deviceInfo ?: @"";
    dic[@"ext"] = _ext ?: @"";
    return dic;
}

- (NSString *)checkSumWith:(NSData *)requestBody time:(NSString *)timeStr nonce:(NSString *)nonce {
    NSString *salt = @"021cc0370d824a51b7c8180485c27b38";
    NSString *requestString = [[NSString alloc] initWithData:requestBody encoding:NSUTF8StringEncoding];
    NSString *source = [NSString stringWithFormat:@"%@%@%@%@",requestString,nonce,timeStr,salt];
    NSString *checkSum = [self sha1:source];
    return checkSum;
}

- (NSString *)sha1:(NSString *)source {
    NSData *data = [source dataUsingEncoding:NSUTF8StringEncoding];
    uint8_t digest[CC_SHA1_DIGEST_LENGTH];
    CC_SHA1(data.bytes, (CC_LONG)data.length, digest);
    NSMutableString *output = [NSMutableString stringWithCapacity:CC_SHA1_DIGEST_LENGTH * 2];
    for (int i = 0; i < CC_SHA1_DIGEST_LENGTH; i++) {
        [output appendFormat:@"%02x", digest[i]];
    }
    return output;
}

- (NSString *)description {
    return [[self dictionaryValue] description];
}

- (NSMutableURLRequest *)request {
    NSURL *url = [NSURL URLWithString:[self path]];
    NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL:url];
    [req setHTTPMethod:@"POST"];
    [req setValue:@"application/json;charset=utf-8" forHTTPHeaderField:@"Content-Type"];

    NSDictionary *requestDic = [self dictionaryValue];
    NSData *postData = [NSJSONSerialization dataWithJSONObject:requestDic options:0 error:nil];
    NSString *timeStr = [NSString stringWithFormat:@"%ld",(long)([[NSDate date] timeIntervalSince1970] * 1000)];
    NSString *nonce = [NSString stringWithFormat:@"%ld",random()];
    [req addValue:[self checkSumWith:postData time:timeStr nonce:nonce] forHTTPHeaderField:@"CheckSum"];
    [req addValue:nonce forHTTPHeaderField:@"Nonce"];
    [req addValue:timeStr forHTTPHeaderField:@"CurTime"];
    req.HTTPBody = postData;
    return req;
}

- (void)response:(id)responseData error:(NSError *)error {
    
    NIMFNosToken *token = nil;
    if ([responseData isKindOfClass:[NSDictionary class]]) {
        NSDictionary *dataDict = [responseData objectForKey:@"data"];
        token = [[NIMFNosToken alloc] initWithDictionary:dataDict];
    }
    
    if (!token && !error) {
        error = [NSError errorWithDomain:@"nimf.network.gettoken"
                                    code:-1
                                userInfo:@{NSLocalizedDescriptionKey : @"token is nil"}];
    }
    
    if (_completion) {
        _completion(error, token);
    }
}

- (void)resume {
    [NIMFNosTokenManager request:self];
}

@end
