//
//  NIMFNOSHttpManager.m
//  NOSSDK
//
//  Created by hzzhenghuabin on 2015/1/12.
//  Copyright (c) 2015年 NetEase. All rights reserved.
//

#import "NIMFAFNetworking.h"

#import "NIMFNOSConfig.h"
#import "NIMFNOSHttpManager.h"
#import "NIMFNOSUserAgent.h"
#import "NIMFNOSResponseInfo.h"
#import "NIMFNOSUploadManager+Private.h"

@interface NIMFNOSHttpManager ()
@property (nonatomic) NIMFAFHTTPRequestOperationManager *httpManager;
@end

static NSString *userAgent = nil;

@implementation NIMFNOSHttpManager

+ (void)initialize {
	userAgent = NIMFNOSUserAgent();
}

- (instancetype)init {
	if (self = [super init]) {
		_httpManager = [[NIMFAFHTTPRequestOperationManager alloc] init];
		_httpManager.responseSerializer = [NIMFNOSAFJSONResponseSerializer serializer];
        
        // lbs server return json with uncorrect Content-Type(not text/json or application/json), we sadly have to add
        // the following statement. took much time to find the problem and its solution
        _httpManager.responseSerializer.acceptableContentTypes = nil;
        
        NIMFAFSecurityPolicy *securityPolicy = [[NIMFAFSecurityPolicy alloc] init];
        //[securityPolicy setAllowInvalidCertificates:YES];
        [_httpManager setSecurityPolicy:securityPolicy];
	}

	return self;
}

+ (NIMFNOSResponseInfo *)buildResponseInfo:(NIMFAFHTTPRequestOperation *)operation
                            withError:(NSError *)error
                         withResponse:(id)responseObject {
	NIMFNOSResponseInfo *info;
	if (operation.response) {
		int status =  (int)[operation.response statusCode];
		info = [[NIMFNOSResponseInfo alloc] init:status withBody:responseObject];
	}
	else {
		info = [NIMFNOSResponseInfo responseInfoWithNetError:error];
	}
	return info;
}

- (void)  sendRequest:(NSMutableURLRequest *)request
    withCompleteBlock:(NIMFNOSCompleteBlock)completeBlock
    withProgressBlock:(NIMFNOSInternalProgressBlock)progressBlock {
	NIMFAFHTTPRequestOperation *operation = [_httpManager
	                                     HTTPRequestOperationWithRequest:request
	                                                             success: ^(NIMFAFHTTPRequestOperation *operation, id responseObject) {
	    NIMFNOSResponseInfo *info = [NIMFNOSHttpManager buildResponseInfo:operation withError:nil withResponse:operation.responseData];
	    NSDictionary *resp = nil;
	    if (info.isOK) {
	        resp = responseObject;
		}
	    completeBlock(info, resp);
	}                                                            failure: ^(NIMFAFHTTPRequestOperation *operation, NSError *error) {
	    NIMFNOSResponseInfo *info = [NIMFNOSHttpManager buildResponseInfo:operation withError:error withResponse:operation.responseData];
	    completeBlock(info, nil);
	}];

	if (progressBlock) {
		[operation setUploadProgressBlock: ^(NSUInteger bytesWritten, long long totalBytesWritten, long long totalBytesExpectedToWrite) {
		    progressBlock(totalBytesWritten, totalBytesExpectedToWrite);
		}];
	}
	
	[_httpManager.operationQueue addOperation:operation];
}

- (void)         post:(NSString *)url
             withData:(NSData *)data
           withParams:(NSDictionary *)params
          withHeaders:(NSDictionary *)headers
    withCompleteBlock:(NIMFNOSCompleteBlock)completeBlock
    withProgressBlock:(NIMFNOSInternalProgressBlock)progressBlock
      withCancelBlock:(NIMFNOSCancelBlock)cancelBlock {
	NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:[[NSURL alloc] initWithString:url]];
	if (headers) {
		[request setAllHTTPHeaderFields:headers];
	}
    [request setValue:userAgent forHTTPHeaderField:@"User-Agent"];

	[request setHTTPMethod:@"POST"];

	if (params) {
		[request setValuesForKeysWithDictionary:params];
	}
    
    if (data) {
        [request setHTTPBody:data];
    }
    
    // have not found a way to set connection time out, will try later.
    [request setTimeoutInterval:[[NIMFNOSUploadManager globalConf] NOSSoTimeout]];
    
	[self       sendRequest:request
          withCompleteBlock:completeBlock
          withProgressBlock:progressBlock];
}

- (void)         get:(NSString *)url
          withParams:(NSDictionary *)params
         withHeaders:(NSDictionary *)headers
   withCompleteBlock:(NIMFNOSCompleteBlock)completeBlock {
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:[[NSURL alloc] initWithString:url]];
    if (headers) {
        [request setAllHTTPHeaderFields:headers];
    }
    [request setValue:userAgent forHTTPHeaderField:@"User-Agent"];
    
    [request setHTTPMethod:@"GET"];
    
    if (params) {
        [request setValuesForKeysWithDictionary:params];
    }
    
    // have not found a way to set connection time out, will try later.
    [request setTimeoutInterval:[[NIMFNOSUploadManager globalConf] NOSSoTimeout]];
    
    [self       sendRequest:request
          withCompleteBlock:completeBlock
          withProgressBlock:nil];}

@end
