//
//  NIMFNOSLbsNetMonitorTask.m
//  NOSSDK
//
//  Created by hzzhenghuabin on 2015/1/17.
//  Copyright (c) 2015年 NetEase. All rights reserved.
//

#import "NIMFNOSLbsNetMonitorTask.h"
#import "NIMFReachability.h"
#import "NIMFNOSUploadManager+Private.h"

@interface NIMFNOSLbsNetMonitorTask()

@property (nonatomic) NIMFNetworkStatus status;
@property (nonatomic) NIMFReachability *internetReachability;

@end

@implementation NIMFNOSLbsNetMonitorTask

+ (instancetype)sharedInstanceAndRunWithCompleteHandler:(NIMFNOSLbsCompletionHandler) completeHandler {
    static NIMFNOSLbsNetMonitorTask *sharedInstance = nil;
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [[self alloc] initWithCompleteHandler:completeHandler];
    });
    
    return sharedInstance;
}

- (instancetype) initWithCompleteHandler:(NIMFNOSLbsCompletionHandler) completeHandler {
    if (self = [super initWithCompleteHandler:completeHandler]) {
        _status = NIMFNotReachable;
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:NIMFReachabilityChangedNotification object:nil];
        
        // use internet connection for now, if test fails, I'll try others
        _internetReachability = [NIMFReachability reachabilityForInternetConnection];
        [_internetReachability startNotifier];
        [self updateUploaderHostsWithReachability:_internetReachability];
    }
    return self;
}

- (void) reachabilityChanged:(NSNotification *)note
{
    NIMFReachability* curReach = [note object];
    [self updateUploaderHostsWithReachability:curReach];
}

- (void)updateUploaderHostsWithReachability:(NIMFReachability *)reachability {
    if (reachability == self.internetReachability) {
        NIMFNetworkStatus netStatus = [reachability currentReachabilityStatus];
        
        switch (netStatus)
        {
            case NIMFNotReachable: {
                break;
            }
                
            case NIMFReachableViaWWAN: {
                if (self.status != netStatus) {
                    @autoreleasepool {
                        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                            //[self fetchUploaderHosts:@"3g"];
                            [[NIMFNOSUploadManager bucketLbsMap] invalidLBSInfo];
                        });
                    }
                }
                self.status = netStatus;
                [NIMFNOSUploadManager bucketLbsMap].net = @"3g";
                break;
            }
            case NIMFReachableViaWiFi: {
                if (self.status != netStatus) {
                    @autoreleasepool {
                        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                            //[self fetchUploaderHosts:@"wifi"];
                            [[NIMFNOSUploadManager bucketLbsMap] invalidLBSInfo];
                        });
                    }
                }
                self.status = netStatus;
                [NIMFNOSUploadManager bucketLbsMap].net = @"wifi";
                break;
            }
        }
    }
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NIMFReachabilityChangedNotification object:nil];
}

@end
