//
//  NIMFNOSLbsTask.m
//  NOSSDK
//
//  Created by hzzhenghuabin on 2015/1/17.
//  Copyright (c) 2015年 NetEase. All rights reserved.
//

#import "NIMFNOSLbsTask.h"
#import "NIMFNOSResponseInfo.h"
#import "NIMFNOSUploadManager+Private.h"
#import "NIMFNOSTimeUtils.h"
#import "NIMFNOSBucketLBSMap.h"

@interface NIMFNOSLbsTask()

@property (nonatomic, strong) NIMFNOSLbsCompletionHandler completeHandler;

@end

extern int const KNIMFNOSLbsHostNum;
extern NSString const *kNIMFNOSLbsHostArray[];
extern NSString *const kNIMFNOSLbsVersion;

@implementation NIMFNOSLbsTask

- (instancetype) initWithCompleteHandler:(NIMFNOSLbsCompletionHandler) completeHandler {
    if (self = [super init]) {
        _completeHandler = completeHandler;
    }
    return self;
}

- (void) fetchUploaderHosts:(NSString *)bucketName {
    // get from cache
    NIMFNOSLbsInfo *lbsInfo = [[NIMFNOSUploadManager bucketLbsMap] lbsInfoSnapshot:bucketName] ;
    NSMutableArray *lbsHosts = [NSMutableArray arrayWithCapacity:KNIMFNOSLbsHostNum + 1];
    
    /*if (lbsInfo && lbsInfo.lbsIP) {
        if ([lbsInfo.lbsIP hasPrefix:@"http://"]) {
            NSMutableString * url = [NSMutableString stringWithString:@"https://"];
            [url appendString: [lbsInfo.lbsIP substringFromIndex:[@"http://" length]]];
            [lbsHosts addObject:url];
        }
        else {
            [lbsHosts addObject:lbsInfo.lbsIP];
        }
    }*/
    
    if ([NIMFNOSUploadManager globalConf].NOSLbsHostArray.count != 0) {
        [lbsHosts addObjectsFromArray:[NIMFNOSUploadManager globalConf].NOSLbsHostArray];
    } else {
        for (int i = 0; i < KNIMFNOSLbsHostNum; ++i) {
            [lbsHosts addObject:kNIMFNOSLbsHostArray[i]];
        }
    }
    
    long long startTime = [NIMFNOSTimeUtils currentMil];
    
    NSURLResponse *response;
    NSError *error;
    
    for (NSString *host in lbsHosts) {
        NSMutableString * lbsUrl = [NSMutableString stringWithString:host];
        [lbsUrl appendString:@"?"];
        [lbsUrl appendString:kNIMFNOSLbsVersion];
        [lbsUrl appendString:@"&bucketname="];
        [lbsUrl appendString:bucketName];
        
        NSURL *url = [[NSURL alloc] initWithString:lbsUrl];
        NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:url];
        [request setHTTPMethod:@"GET"];
        [request setTimeoutInterval:[[NIMFNOSUploadManager globalConf] NOSSoTimeout]];
        
        UInt32 retriedTimes = 0;//[[NIMFNOSUploadManager globalConf] NOSRetryCount];
        
        while (true) {
            response = nil;
            error = nil;
            NSData *result = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
            
            if (error) {
                NSString *errorDesc = [error localizedDescription];
                NSLog(@"fetch uploader host from lbs error: %@", errorDesc);
                //NSLog(@"%d", [NIMFNOSUploadManager globalConf].NOSRetryCount);
                if (retriedTimes >= [NIMFNOSUploadManager globalConf].NOSRetryCount) {
                    break;
                }
                retriedTimes++;
            } else {
                NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *)response;
                int statusCode = (int)[httpResponse statusCode];
                
                if (statusCode == 200) {
                    id jsonObject = [NSJSONSerialization JSONObjectWithData:result
                                                                    options:NSJSONReadingMutableContainers
                                                                      error:&error];
                    if (error) {
                        NSLog(@"lbs return non json");
                        long long endTime = [NIMFNOSTimeUtils currentMil];
                        self.completeHandler(nil, endTime - startTime, statusCode, 1, bucketName);
                        return;
                    }
                    if ([jsonObject isKindOfClass:[NSDictionary class]]) {
                        NSDictionary *jsonDictionary = (NSDictionary*)jsonObject;
                        NSArray* arrayResult = [jsonDictionary valueForKey:@"upload"];
                        NSLog(@"lbs return uploader hosts:%@", arrayResult);
                        long long endTime = [NIMFNOSTimeUtils currentMil];
                        self.completeHandler(jsonDictionary, endTime - startTime, statusCode, 0, bucketName);
                        return;
                    }
                } else {
                    NSLog(@"lbs return %d", statusCode);
                    long long endTime = [NIMFNOSTimeUtils currentMil];
                    self.completeHandler(nil, endTime - startTime, statusCode, 1, bucketName);
                    return;
                }
            } // End of if (error)
        } // End of while
    } // End of For
    
    long long endTime = [NIMFNOSTimeUtils currentMil];
    self.completeHandler(nil, endTime - startTime, -1, 1, bucketName);
    return;
}

@end
