//
//  NOSBucketLBSMap.m
//  NOSSDK
//
//  Created by hzzhenghuabin on 2016/2/29.
//  Copyright (c) 2016年 NetEase. All rights reserved.
//

#import "NIMFNOSBucketLBSMap.h"
#import "NIMFNOSUploadManager+Private.h"
#import "NIMFNOSTimeUtils.h"

static NSString* const kNOSUploadHostsStr = @"uploadHosts";
static NSString* const kNOSLbsIpStr = @"lbsIp";
static NSString* const kNOSLastModified = @"lastModified";
static NSString* const kNOSBucketPrifix = @"NOSBucketName_";


@implementation NIMFNOSBucketLBSMap

- (instancetype)init {
    if (self = [super init]) {
        _bucketLbsMap = [[NSMutableDictionary alloc] init];
        _persistentMap = [NSUserDefaults standardUserDefaults];
    }
    return self;
}

- (void) addLbsInfo:(NIMFNOSLbsInfo*)lbsInfo
          forBucket:(NSString *)bucketName {
    if (!bucketName || !lbsInfo) {
        return;
    }
    
    NSDictionary *dic = [NIMFNOSBucketLBSMap dicWithLbsInfo:lbsInfo];
    
    @synchronized(_bucketLbsMap) {
        //NIMFNOSLbsInfo *temp = [_bucketLbsMap objectForKey:bucketName];
        //if (temp) {
        //    lbsInfo.net = temp.net;
        //}
        [_bucketLbsMap setObject:lbsInfo
                      forKey:bucketName];
        [_persistentMap setObject:dic
                           forKey:[NIMFNOSBucketLBSMap bucketStrInPersistent:bucketName]];
    }
}

+ (NSDictionary *) dicWithLbsInfo:(NIMFNOSLbsInfo*)lbsInfo {
    if (!lbsInfo) {
        return nil;
    }
    
    return [NSDictionary dictionaryWithObjectsAndKeys:lbsInfo.uploaderHosts, kNOSUploadHostsStr,
            lbsInfo.lbsIP, kNOSLbsIpStr, [NSNumber numberWithLongLong:lbsInfo.lastModified], kNOSLastModified, nil];
}

+ (NSString *) bucketStrInPersistent:(NSString *) bucketName {
    return [NSString stringWithFormat:@"%@%@", kNOSBucketPrifix, bucketName];
}

- (void) invalidLBSInfoForBucket:(NSString *)bucketName {
    @synchronized(_bucketLbsMap) {
        NIMFNOSLbsInfo *lbsInfo = [_bucketLbsMap objectForKey:bucketName];
        if (lbsInfo) {
            lbsInfo.lastModified = 0;
            NSDictionary *dic = [NIMFNOSBucketLBSMap dicWithLbsInfo:lbsInfo];
            [_persistentMap setObject:dic
                               forKey:[NIMFNOSBucketLBSMap bucketStrInPersistent:bucketName]];
        }
    }
}

- (void) invalidLBSInfo {
    @synchronized(_bucketLbsMap) {
        for (NSString * bucket in _bucketLbsMap) {
            NIMFNOSLbsInfo *lbsInfo = [_bucketLbsMap objectForKey:bucket];
            lbsInfo.lastModified = 0;
            NSDictionary *dic = [NIMFNOSBucketLBSMap dicWithLbsInfo:lbsInfo];
            [_persistentMap setObject:dic
                               forKey:[NIMFNOSBucketLBSMap bucketStrInPersistent:bucket]];
        }
    }
}

- (NIMFNOSLbsInfo*) lbsInfoSnapshot:(NSString *)bucketName {
    NIMFNOSLbsInfo *lbsInfo = [_bucketLbsMap objectForKey:bucketName];
        
    if (!lbsInfo) {
        NSDictionary *dic = [_persistentMap dictionaryForKey:[NIMFNOSBucketLBSMap bucketStrInPersistent:bucketName]];
        if (dic) {
            lbsInfo = [[NIMFNOSLbsInfo alloc] init];
            lbsInfo.uploaderHosts = [dic objectForKey:kNOSUploadHostsStr];
            lbsInfo.lbsIP = [dic objectForKey:kNOSLbsIpStr];
            lbsInfo.lastModified = [[dic objectForKey:kNOSLastModified] longLongValue];
            lbsInfo.hasPushed = YES;
            [self addLbsInfo:lbsInfo forBucket:bucketName];
        }
    }
    
    if (!lbsInfo) {
        return nil;
    }
        
    NIMFNOSLbsInfo *tempInfo = [lbsInfo clone];
    //lbsInfo.hasPushed = YES;
    return tempInfo;
}

- (NIMFNOSLbsInfo*) lbsInfoNewest:(NSString*)bucketName {
    NIMFNOSLbsInfo *lbsInfo = [self lbsInfoSnapshot:bucketName];
    
    BOOL isNew = NO;
    if (!lbsInfo || lbsInfo.lastModified + 1000 * [[NIMFNOSUploadManager globalConf] NOSRefreshInterval] <= [NIMFNOSTimeUtils currentMil]) {
        NIMFNOSLbsTask* lbsTask = [NIMFNOSUploadManager lbsTask];
        [lbsTask fetchUploaderHosts: bucketName];
        isNew = YES;
    }
    
    NIMFNOSLbsInfo *resLbsInfo = [self lbsInfoSnapshot:bucketName];
    if (isNew) {
        resLbsInfo.hasPushed = NO;
    }
    return resLbsInfo;
}


@end
