//
//  NOSUploader.h
//  NOSSDK
//
//  Modified by hzzhenghuabin on 2015/1/18.
//  Copyright (c) 2015年 NetEase. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "NIMFNOSConfig.h"
#import "NIMFNOSHttpManager.h"
#import "NIMFNOSResponseInfo.h"
#import "NIMFNOSUploadManager.h"
#import "NIMFNOSResumeUpload.h"
#import "NIMFNOSUploadOption+Private.h"
#import "NIMFNOSAsyncRun.h"
#import "NIMFNOSLbsNetMonitorTask.h"
#import "NIMFNOSBucketLBSMap.h"
#import "NIMFNOSStatTimeTask.h"
#import "NIMFNOSTimeUtils.h"
#import "NIMFNOSLbsTask.h"

@interface NIMFNOSUploadManager ()
@property (nonatomic) NIMFNOSHttpManager *httpManager;
@property (nonatomic) id <NIMFNOSRecorderDelegate> recorder;
@property (nonatomic, strong) NIMFNOSRecorderKeyGenerator recorderKeyGen;
@end

/**
 * static, NIMFNOSUploadManager may be multiple instances, 
 * though we recommend user use NIMFNOSUploadManager as a singleton
 */
static NIMFNOSConfig* globalConf;
static NIMFNOSBucketLBSMap* bucketLbsMap;
static NIMFNOSLbsTask* lbsTask;
static NIMFNOSLbsNetMonitorTask* lbsNetMonitorTask;
static NIMFNOSStatTimeTask* statTimeTask;

@implementation NIMFNOSUploadManager

+ (void)initialize {
    globalConf = [[NIMFNOSConfig alloc] init];
    bucketLbsMap = [[NIMFNOSBucketLBSMap alloc] init];
}

+ (void)setGlobalConf:(NIMFNOSConfig*) conf {
    globalConf = conf;
}

+ (NIMFNOSConfig*) globalConf {
    return globalConf;
}

+ (NIMFNOSStatTimeTask*) statTimeTask {
    return statTimeTask;
}

+ (NIMFNOSLbsTask*) lbsTask {
    return lbsTask;
}

+ (NIMFNOSBucketLBSMap*) bucketLbsMap; {
    return bucketLbsMap;
}

- (instancetype)init {
    return [self initWithRecorder:nil recorderKeyGenerator:nil];
}

- (instancetype)initWithRecorder:(id <NIMFNOSRecorderDelegate> )recorder {
    return [self initWithRecorder:recorder recorderKeyGenerator:nil];
}

- (instancetype)initWithRecorder:(id <NIMFNOSRecorderDelegate> )recorder
            recorderKeyGenerator:(NIMFNOSRecorderKeyGenerator)recorderKeyGenerator {
	if (self = [super init]) {
		_httpManager = [[NIMFNOSHttpManager alloc] init];
		_recorder = recorder;
		_recorderKeyGen = recorderKeyGenerator;
        
        NIMFNOSLbsCompletionHandler lbsCompletionHandler = ^(NSDictionary *resp, long long lbsUseTime,
                                                         int lbsHttpCode, int lbsSucc, NSString *bucketname) {
            NSString *lbsIP = nil;
            NSArray *uploaderHosts = nil;
            NIMFNOSLbsInfo *lbsInfo = [bucketLbsMap lbsInfoSnapshot:bucketname];
            
            if (!lbsInfo) {
                lbsInfo = [[NIMFNOSLbsInfo alloc] init];
            }
            
            if (resp) {
                lbsIP = [resp valueForKey:@"lbs"];
                uploaderHosts = [resp valueForKey:@"upload"];
                lbsInfo.lbsIP = lbsIP;
                lbsInfo.uploaderHosts = uploaderHosts;
                lbsInfo.lastModified = [NIMFNOSTimeUtils currentMil];
            }
            
            lbsInfo.lbsUseTime = lbsUseTime;
            lbsInfo.lbsSucc = lbsSucc;
            lbsInfo.lbsHttpCode = lbsHttpCode;
            lbsInfo.hasPushed = YES;
            //if (net) lbsInfo.net = net;
            
            [bucketLbsMap addLbsInfo:lbsInfo forBucket:bucketname];
            
        };
        
        // start lbs
        lbsTask = [[NIMFNOSLbsTask alloc] initWithCompleteHandler:lbsCompletionHandler];
        
        lbsNetMonitorTask = [NIMFNOSLbsNetMonitorTask
                             sharedInstanceAndRunWithCompleteHandler:lbsCompletionHandler];
        
        // start stat
        statTimeTask = [NIMFNOSStatTimeTask sharedInstanceAndRun];
	}
	return self;
}

+ (instancetype)sharedInstanceWithRecorder:(id <NIMFNOSRecorderDelegate> )recorder
                      recorderKeyGenerator:(NIMFNOSRecorderKeyGenerator)recorderKeyGenerator {
	static NIMFNOSUploadManager *sharedInstance = nil;

	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
	    sharedInstance = [[self alloc] initWithRecorder:recorder
                                   recorderKeyGenerator:recorderKeyGenerator
                                             ];
	});

	return sharedInstance;
}

+ (BOOL)checkAndNotifyError:(NSString *)bucket
                        key:(NSString *)key
                      token:(NSString *)token
                       data:(NSData *)data
                       file:(NSString *)file
                   complete:(NIMFNOSUpCompletionHandler)completionHandler {
	NSString *desc = nil;
	if (completionHandler == nil) {
		@throw [NSException exceptionWithName:NSInvalidArgumentException
		                               reason:@"no completionHandler" userInfo:nil];
		return YES;
	}
    if (!bucket || [bucket length] < 1 || !key || [key length] < 1) {
        desc = @"bucket or key is empty";
    }
	if (data == nil && file == nil) {
		desc = @"no input data";
	}
	else if (token == nil || [token isEqualToString:@""]) {
		desc = @"no token";
	}
	if (desc != nil) {
		NIMFNOSAsyncRun ( ^{
		    completionHandler([NIMFNOSResponseInfo responseInfoWithInvalidArgument:desc], key, nil);
		});
		return YES;
	}
	return NO;
}

- (void)putFileByHttp:(NSString *)filePath
               bucket:(NSString*)bucket
                  key:(NSString *)key
                token:(NSString *)token
             complete:(NIMFNOSUpCompletionHandler)completionHandler
               option:(NIMFNOSUploadOption *)option {
    [self putFile:filePath
         protocal:@"http"
           bucket:bucket
              key:key
            token:token
         complete:completionHandler
           option:option];
}

- (void)putFileByHttps:(NSString *)filePath
                bucket:(NSString*)bucket
                   key:(NSString *)key
                 token:(NSString *)token
              complete:(NIMFNOSUpCompletionHandler)completionHandler
                option:(NIMFNOSUploadOption *)option {
    [self putFile:filePath
         protocal:@"https"
           bucket:bucket
              key:key
            token:token
         complete:completionHandler
           option:option];
    
}

- (void)putFile:(NSString *)filePath
       protocal:(NSString *)protocal
         bucket:(NSString *)bucket
            key:(NSString *)key
          token:(NSString *)token
       complete:(NIMFNOSUpCompletionHandler)completionHandler
         option:(NIMFNOSUploadOption *)option {
             if ([NIMFNOSUploadManager checkAndNotifyError:bucket key:key token:token data:nil file:filePath complete:completionHandler]) {
		return;
	}

	@autoreleasepool {
		NSError *error = nil;
		NSDictionary *fileAttr = [[NSFileManager defaultManager] attributesOfItemAtPath:filePath error:&error];

		if (error) {
			NIMFNOSAsyncRun ( ^{
			    NIMFNOSResponseInfo *info = [NIMFNOSResponseInfo responseInfoWithFileError:error];
			    completionHandler(info, key, nil);
			});
			return;
		}

		NSNumber *fileSizeNumber = fileAttr[NSFileSize];
		UInt32 fileSize = [fileSizeNumber intValue];
		NSData *data = [NSData dataWithContentsOfFile:filePath options:NSDataReadingMappedIfSafe error:&error];
		if (error) {
			NIMFNOSAsyncRun ( ^{
			    NIMFNOSResponseInfo *info = [NIMFNOSResponseInfo responseInfoWithFileError:error];
			    completionHandler(info, key, nil);
			});
			return;
		}

		NIMFNOSUpCompletionHandler complete = ^(NIMFNOSResponseInfo *info, NSString *key, NSDictionary *resp)
		{
			completionHandler(info, key, resp);
		};

		NSDate *modifyTime = fileAttr[NSFileModificationDate];
        NSString *recorderKey = key;
        if (_recorder != nil && _recorderKeyGen != nil) {
            recorderKey = _recorderKeyGen(key, filePath);
        }
        
        NIMFNOSResumeUpload *up = [[NIMFNOSResumeUpload alloc]
                                  initWithData:data
                                  withSize:fileSize
                                  withProtocal:protocal
                                  withBucket:bucket
                                  withKey:key
                                  withToken:token
                                  withCompletionHandler:complete
                                  withOption:option
                                  withModifyTime:modifyTime
                                  withRecorder:_recorder
                                  withRecorderKey:recorderKey
                                  withHttpManager:_httpManager];
        NIMFNOSAsyncRun ( ^{
            [up run];
		});
	}
}

@end
