//
//  NIMNOSUploadTask.m
//  YixinCall
//
//  Created by amao on 12/26/14.
//  Copyright (c) 2014 amao. All rights reserved.
//

#import "NIMFNOSUploadTask.h"
#import "NIMFNOSUtil.h"
#import "NSString+NIMF.h"
#import "NIMFNOSUploadManager.h"
#import "NIMFNosToken.h"
#import "NIMFNOSUploadOption.h"
#import "NIMFNOSResponseInfo.h"

static NSString *NIMFBucketPlaceholder = @"{bucket}";
static NSString *NIMFObjectPlaceholder = @"{object}";

@interface NIMFNOSUploadTask ()

//上传信息
@property (nonatomic,copy)      NSString *filepath;
@property (nonatomic,strong)    NIMFNosToken *token;
@property (nonatomic,copy)      NIMFHttpCompletionBlock callback;
@property (nonatomic,copy)      NIMFHttpProgressBlock progress;

//上传中信息
@property (nonatomic,assign)    unsigned long long fileSize;
@property (nonatomic,strong)    NSFileHandle  *fileHandle;
@property (nonatomic,copy)      NSString *contentType;

//上传结果
@property (nonatomic,copy)      NSString *urlString;
@property (nonatomic,strong)    NSError  *nosError;

//sdk上传管理
@property (nonatomic,readonly) NIMFNOSUploadManager *uploadManager;
@property (nonatomic,strong) dispatch_semaphore_t syncSem;

@property (nonatomic,copy) NSString *nosDownloadAddress;
@property (nonatomic,assign) BOOL httpsEnabled;

@end

@implementation NIMFNOSUploadTask
+ (instancetype)task:(NSString *)filepath
               token:(NIMFNosToken *)token
            progress:(NIMFHttpProgressBlock)progress
          completion:(NIMFHttpCompletionBlock)completion
{
    NIMFNOSUploadTask *task = [[NIMFNOSUploadTask alloc] init];
    task.filepath   = filepath;
    task.token      = token;
    task.progress   = progress;
    task.callback   = completion;
    return task;
}

- (instancetype)init
{
    if (self = [super init])
    {
        _httpsEnabled = YES;
        _nosDownloadAddress = [NSString stringWithFormat:@"%@-nosdn.netease.im/%@",
                               NIMFBucketPlaceholder,NIMFObjectPlaceholder];
        _syncSem = dispatch_semaphore_create(0);
    }
    return self;

}

- (void)main
{
    if ([self isParamValid])
    {
        {
            [self doUploadUseSdk]; //使用Nos Sdk进行上传
        }
    }
    else
    {
        NSLog(@"invalid upload param %@ error %@",_filepath,_nosError);
    }
    [self raiseCallback];
}

#pragma mark - 检查上传参数
+ (NSError *)errorWithCode:(int)code description:(NSString *)description {
    NSDictionary *userInfo = nil;
    if (description.length != 0) {
        userInfo = @{NSLocalizedDescriptionKey : description};
    }
    NSError *error = [NSError errorWithDomain:@"NIMFLocalErrorDomain"
                                         code:code
                                     userInfo:userInfo];
    return error;
}

- (BOOL)isParamValid
{
    BOOL valid = YES;
    
    do
    {
        if (![[NSFileManager defaultManager] fileExistsAtPath:_filepath] ||
            (_fileSize = [NIMFNOSUtil fileSize:_filepath]) == 0)
        {
            _nosError = [NIMFNOSUploadTask errorWithCode:-1 description:@"file not exist!"];
            valid = NO;
            break;
        }
        else
        {
            _fileHandle = [NSFileHandle fileHandleForReadingAtPath:_filepath];
            if (_fileHandle == nil)
            {
                _nosError = [NIMFNOSUploadTask errorWithCode:-1 description:@"file can't be read!"];
                valid = NO;
                break;
            }
        }
        
        if (![_token isParamValid]) {
            _nosError = [NIMFNOSUploadTask errorWithCode:-2 description:@"InValid token!"];
            valid = NO;
            break;
        }
        
        _contentType = [NIMFNOSUtil mimeType:_filepath];
        
        NSLog(@"check file info %@\n\ntoken %@\n\ncontent type %@",_filepath,_token,_contentType);
        
    }
    while (0);
    return valid;
}

#pragma mark - 上传过程
- (void)doUploadUseSdk {
    NSString *filePath = _filepath;
    NSString *bucket = [_token bucket];
    NSString *key = [_token objectName];
    NSString *token = [_token token];
    __weak typeof(self) weakSelf = self;
        
    //进度
    NOSUpProgressHandler processHandler = ^(NSString *key, float percent) {
        [weakSelf raiseProgressCallbackWithProgress:percent];
    };
    
    //取消
    NOSUpCancellationSignal cancelHandler = ^BOOL{
        return [weakSelf isCancelled];
    };
    
    NIMFNOSUploadOption *option = [[NIMFNOSUploadOption alloc] initWithMime:nil
                                                            progressHandler:processHandler
                                                                      metas:nil
                                                         cancellationSignal:cancelHandler];
    
    //上传
    [self.uploadManager putFileByHttps:filePath
                                bucket:bucket
                                   key:key
                                 token:token
                              complete:^(NIMFNOSResponseInfo *info, NSString *key, NSDictionary *resp) {
        __strong typeof(weakSelf) strongSelf = weakSelf;
        if (info.isOK) {
            [strongSelf generateURL];
        } else {
            strongSelf.nosError = [NIMFNOSUploadTask errorWithCode:info.statusCode description:@"Http request failed!"];
            NSLog(@"upload %@ failed! requestId:%@ break:%d cancelled %d error:%@",
                      [strongSelf.filepath lastPathComponent],
                      info.requestID,
                      info.broken,
                      info.isCancelled,
                      info.error);
            if (info.isCancelled)
            {
                strongSelf.nosError = [NIMFNOSUploadTask errorWithCode:0 description:@"Upload Cancelled!"];
            }
        }
        dispatch_semaphore_signal(strongSelf.syncSem);
    } option:option];
    
    //这个task里需要同步执行才能保证生命周期，因此这里等一下。
    dispatch_semaphore_wait(_syncSem, DISPATCH_TIME_FOREVER);
}

- (void)generateURL
{
//    NSString *scheme = _httpsEnabled ? @"https" : @"http";
//    NSString *downloadAddress = _nosDownloadAddress;
//    NSString *bucketName = [_token bucket];
//    NSString *objectName = [_token objectName];
//    if ([bucketName length] && [objectName length])
//    {
//        downloadAddress = [downloadAddress stringByReplacingOccurrencesOfString:NIMFBucketPlaceholder
//                                                                     withString:bucketName];
//        downloadAddress = [downloadAddress stringByReplacingOccurrencesOfString:NIMFObjectPlaceholder
//                                                                     withString:objectName];
//    }
//    else
//    {
//        NSLog(@"invalid token: bucket %@ object %@",bucketName,objectName);
//    }
//
//
//    NSString *urlString = [NSString stringWithFormat:@"%@://%@", scheme, downloadAddress ? : @""];
    
//    NSString *survivalTimeString = [NSString stringWithFormat:@"%lld",(long long)[self.token.survivalTime longLongValue]*24*3600];
//
//    urlString = [[NIMNOSSceneManager sharedInstance] nosDownloadAddress:urlString survivalTime:survivalTimeString];
    
//    _urlString = [urlString nimf_urlDecodeString];//梁叔把token的objectNameEncode了一把(为了兼容NOS).. 这里需要Decode回来
    
    _urlString = [_token objectName];
    NSLog(@"nosDownloadAddress %@",_urlString);
    
    _nosError  = nil;
}

#pragma mark - 回调
- (void)raiseCallback
{
    NSLog(@"raise callback file %@ url %@ error %@",[_filepath lastPathComponent],_urlString,_nosError);
    dispatch_async(dispatch_get_main_queue(), ^{
        if (_callback)
        {
            _callback(_nosError, _urlString);
        }
    });
}

- (void)raiseProgressCallbackWithProgress:(CGFloat)progress {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (_progress)
        {
            _progress(progress);
        }
    });
}

- (NIMFNOSUploadManager *)uploadManager {
    NIMFNOSUploadManager *manager = [NIMFNOSUploadManager sharedInstanceWithRecorder:nil
                                                                recorderKeyGenerator:nil];
    return manager;
}


@end
