/*
 *  Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 *  Use of this source code is governed by a MIT license that can be found in the LICENSE file
 */

part of nos;

typedef Callback<T> = void Function(int code, T data);

///提供文件上传功能，采用单例模式
class NosPlugin {
  static NosPlugin? _instance;

  factory NosPlugin() {
    if (_instance == null) {
      final MethodChannel _channel = const MethodChannel('nos_sdk');
      _instance = NosPlugin.private(_channel);
    }
    return _instance!;
  }

  final MethodChannel _methodChannel;

  final String receiptIdKey = "receiptId";

  static int initId = 0;

  Map<String, dynamic> callbacks = Map<String, dynamic>();

  NosPlugin.private(this._methodChannel) {
    _methodChannel.setMethodCallHandler(_handler);
  }

  dynamic _queryCallback({required Map map, bool oneShot = true}) {
    final receiptId = (map[receiptIdKey] ?? '') as String;
    print("_queryCallback key=" + receiptId);
    if (callbacks.containsKey(receiptId)) {
      if (oneShot) {
        return callbacks.remove(receiptId);
      } else {
        return callbacks[receiptId];
      }
    }
    return null;
  }

  void deleteCallback(Map map) {
    String receiptId = map != null ? map[receiptIdKey] : null;
    print("_queryCallback key=" + receiptId);
    if (callbacks.containsKey(receiptId)) {
      callbacks.remove(receiptId);
    }
  }

  // inner use 
  String _generateReceiptId() {
    initId++;
    return "nos_" + initId.toString();
  }

  ///参数转换方法
  Map<String, dynamic> buildArguments<T>({Map<String, dynamic>? arg, Callback<T>? callback}) {
    Map<String, dynamic> map = {};
    if (callback != null) {
      var receiptId = _generateReceiptId();
      callbacks[receiptId] = callback;
      map[receiptIdKey] = receiptId;
    }
    if (arg != null && arg.isNotEmpty) {
      map.addAll(arg);
    }
    return map;
  }

  ///文件上传功能
  Future<void> uploadSync(String sdkType, String sdkVersionName,
      String path,
      String appKey,
      Callback<NosUploadState> callback) async {
    Map map = buildArguments(callback: callback);
    map['sdkType'] = sdkType;
    map['sdkVersionName'] = sdkVersionName;
    map['path'] = path;
    map['appKey'] = appKey;
    return await _methodChannel.invokeMethod("uploadSync", map);
  }

  /// native  --  dart
  Future<dynamic> _handler(MethodCall call) {
    if (call.arguments is Map) {
      dynamic callback;
      var arguments = call.arguments as Map;
      switch (call.method) {
        case 'onUploadSync':
          callback = _queryCallback(map: arguments, oneShot: false);
          _uploadSync(arguments, callback);
          break;
      }
    }
    return Future.value();
  }

  void _uploadSync(Map<dynamic, dynamic> arg, callback) {
    if (callback == null) {
      return;
    }
    NosUploadState state = NosUploadState._fromJson(arg);
    if (state.state != UploadState.uploading) {
      deleteCallback(arg);
    }
    callback(200, state);
  }
}
